<?php
/* EatingEngine E2 - A flexible, modular web framework
    Copyright (C)   2005   Thomas Gabor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

class Core_Drivers_Driver extends Core_Driver 
    implements Core_DriverBindings
{

    protected $Driver = null;
    protected $RequestedClass = "";
    
    protected function construct()
    {
        //save the requested class as driver receiver
        $this->RequestedClass = $this->getFirstParam
                (array("Class", "RequestedClass", "ClassName"));
        //check the given source
        /*switch ( gettype($this->getSource()) )
        {
            case "array":
                $Source = $this->getSource();
                break;
            default:
                $Source = array($this->getSource());
        };*/
        
        //load the configuration file
        $Config = $this->getDriver
            (
                "Core", 
                "ArrayConfig", 
                $this->getPart(), 
                array("Config", "Drivers")
            );
        //search for an appropiate driver
        $Driver = $this->loadDriver
            (
                $this->getDriverByConfig($Config->getDeepTable())
            );
        //check if the operation was successful
        if ( is_object($Driver) )
        { 
            $this->Driver = $Driver;
            $this->succeed();
            return;
        };    
        
        //load the config from Core and do the same thing
        $Config = $this->getDriver
            ("Core", "ArrayConfig", "Core", array("Config", "Drivers"));
        //search for an appropiate driver
        $Driver = $this->loadDriver
            (
                $this->getDriverByConfig($Config->getDeepTable())
            );
        //check if the operation was successful
        if ( is_object($Driver) )
        { 
            $this->Driver = $Driver;
            $this->succeed();
            return;
        };   
        //search for a driver which can be used...
        /*      
        //load all drivers
        $Drivers = $this->getHarddriver("Core", "Drivers");
        //check all drivers   
        foreach ( $Drivers->getDrivers() as $Number => $Driver )
        {
            if ( is_object($this->loadDriver($Driver)) )
            {
                if ( $Driver->getSuccess() === true )
                {
                    //check if we know the class
                    if 
                    ( 
                        is_string($this->RequestedClass) 
                        && (!empty($this->RequestedClass)) 
                    )
                    { 
                        //check if the driver is accepted 
                        if 
                        ( 
                            call_user_func
                                (
                                    array
                                        (
                                        $this->RequestedClass,
                                        "isDriverSuitable"
                                        ),
                                    $Driver 
                                )
                            === true
                        )
                        {
                            //save the driver and exit the foreach
                            $this->Driver = $Driver;
                            $this->succeed();
                            break;
                        };
                    }
                    else
                    {
                        //save the driver and exit the foreach
                        $this->Driver = $Driver;
                        $this->succeed();
                        break;
                    };
                };
            };
        };
        */
        
    }
    
    public function getObject()
    {
        return $this->Driver;
    }
    
    public function getText()
    {
        return get_class($this->Driver);
    }
    
    public function getList()
    {
        return array($this->Driver);
    }
    
    public function getTable()
    {
        return array($this->Driver);
    }
    
    public function getDeepList()
    {
        return array($this->Driver);
    }


    public function getDeepTable()
    {
        return array($this->Driver);
    }
    
    public function getReport()
    {
        return array 
            (
                "Driver" => $this->Driver,
                "RequestedClass" => $this->RequestedClass
            );
    }
    
    public function set($Reiceiver)
    {
        //not possible because this is something virtual
        // it could edit the configuration it uses but it SHOULDN'T!
        // it could edit the actual driver it's representing, but then
        //    you might as well create a new instance...
    }
    
    
    protected function getDriverByConfig($Config)
    {
        //go through configuration (deeper and deeper)
        $ActualSource = $Config;
        foreach ( $this->getEntireSource() as $Number => $Source )
        {
            if ( is_array($ActualSource) )
            {   
                if ( isset($ActualSource[$Source]) )
                {   
                    $ActualSource = $ActualSource[$Source];
                }
                elseif ( isset($ActualSource["default"] ) )
                {
                    $ActualSource = $ActualSource["default"];
                };
            }; 
            if ( is_string($ActualSource) )
            {
                return $ActualSource;
            };
        };
        return false;
    }
    
    protected function loadDriver($DriverName)
    { 
        if ( !class_exists($DriverName) )
        {
            return false;
        };
        $Driver = new $DriverName
            (
                $this->getPart(), 
                $this->getEntireSource(),
                $this->getParams()
            );
        //check if the driver is correct
        if 
        (
            Core_LibClass::implementsIface($Driver, "Core_DriverBindings")
            && Core_LibClass::extendsClass($Driver, "Core_Driver")
        ) 
        {
            return $Driver;
        };
        return false;
    }





}

 ?>