<?php

class Core_Drivers_Parameters extends Core_Driver 
    implements Core_DriverBindings
{

    protected $RawParams = array();

    protected $Params = array
        (
            "super" => array(), 
            "screen" => array(),
            "load" => array()
        );

    protected function construct()
    {
        //load all params
        $this->RawParams = $this->getHarddriver("Core", "Parameters");
        //load all params into categories
        $Categories = array
            (
                "super" => array(),
                "superparam" => array(),
                "screen" => array(),
                "screenparam" => array(),
                "load" => array(),
                "loadparam" => array()
            );
        foreach ( $this->RawParams->getRequest() as $Name => $Value )
        {
            $ParamType = $this->getParamType($Name);
            if ( empty($ParamType) )
            {
                continue;
            };
            if ( !isset($Categories[$ParamType]) )
            {
                $Categories[$ParamType] = array();
            };
            $Categories[$ParamType][$Name] = $Value;
        };
        //build the param tree:
        // load both super and load params by going through this array
        foreach
        ( 
            array
                (
                    "load" => "loadparam", 
                    "super" => "superparam",
                    "screen" => "screenparam"
                )
            as $Group => $ParamType
        )
        { 
            //for each type, load all the http-parameters
            foreach ( $Categories[$Group] as $Name => $Value )
            {
                //create entry of $this->Params
                $GroupName = $this->getGroupName($Name, $Group);
                $this->Params[$Group][$GroupName] = array
                    ("value" => array(), "params" => array());
                //add the params
                foreach 
                ( 
                    $this->getParameters
                        ($GroupName, $Categories[$ParamType], $ParamType)
                    as $ParamName => $ParamValue 
                )
                {
                    $this->Params
                        [$Group]
                        [$GroupName]
                        ["params"]
                        [$this->getParamName($ParamName, $ParamType)]
                        = $ParamValue;
                };
                //save the module
                $this->Params[$Group][$GroupName]["value"] = $Value;
            };
        };
        $this->succeed();
        
        // The way the 'param tree' should look like
        // $this->Params
        //          ["super"]
        //                  ["NameOfSomeSuperHTTPParam"]
        //                          ["value"]  
        //                                  "ValueOfTheSuperHTTPParam" 
        //                          ["params"]
        //                                  ["NameOfSomeSuperSub-Param"]
        //                                          "ValueOfThat"
        //                                  ...
        //                  ...
        //          ["screen"]
        //                  ["PartOfScreen_SimpleScreenName"]
        //                          ["value"]
        //                                  "0"
        //                          ["param"]
        //                                  ["NameOfSomeParamForTheScreen"]
        //                                          "ValueOfThat"
        //                                  ...
        //                  ...
        //          ["load"]
        //                  [0]
        //                          ["value"]
        //                                  "ValueOfTheHTTPParamForLoadingAMod"
        //                          ["params"]
        //                                  ["NameOfSomeSubParamForTheMod"]
        //                                          "ValueOfThat"
        //                                  ...
        //                  ...
    
    }
       
    public function getObject()
    {
        return new Core_Table($this->getDeepTable());
    }
    
    public function getText()
    {
        //no good output possible yet
        return "";
    }
    
    public function getList()
    {
        return $this->RawParams->getRequest();
    }
    
    public function getTable()
    {
        //no good output possible yet
        return "";
    }
    
    public function getDeepList()
    {
        return $this->getList();;
    }


    public function getDeepTable()
    {
        return $this->Params;
    }
  
    public function set($Receiver)
    {
        //actually, there's no reason in editing parameters...
    }
    
    
    
    

    protected function getParamType($Param)
    {
        if 
        ( 
            preg_match("/".Core_LibsysParams::getLoadPattern()."/", $Param) 
        )
        {
            return "load";
        };
        if 
        ( 
            preg_match
                ("/".Core_LibsysParams::getLoadParamPattern()."/", $Param) 
        )
        {
            return "loadparam";
        };
        if
        ( 
            preg_match("/".Core_LibsysParams::getScreenPattern()."/", $Param) 
        )
        {
            return "screen";
        };
        if 
        ( 
            preg_match
                ("/".Core_LibsysParams::getScreenParamPattern()."/", $Param) 
        )
        { 
            return "screenparam";
        };
        if
        ( 
            preg_match("/".Core_LibsysParams::getSuperPattern()."/", $Param) 
        )
        {
            return "super";
        };
        if 
        ( 
            preg_match
                ("/".Core_LibsysParams::getSuperParamPattern()."/", $Param) 
        )
        {
            return "superparam";
        };
        return "";
    }
    
    protected function getValueType($Value)
    {
        if ( preg_match("/".Core_LibsysParams::getPartPattern()."/", $Value) )
        {
            return "part";
        };
        if ( preg_match("/".Core_LibsysParams::getModPattern()."/", $Value) )
        {
            return "mod";
        };
        return "";
    }
       
    
    
    
    
    
    protected function getGroupName($Param, $Type)
    {
        switch ( strtolower($Type) )
        {
            case "super":
                return preg_replace
                    (
                        "/".Core_LibsysParams::getSuperPattern()."/",
                        "\\1",
                        $Param
                    );
            case "superparam":
                return preg_replace
                    (
                        "/".Core_LibsysParams::getSuperParamPattern()."/",
                        "\\1",
                        $Param
                    );
            case "screen":
                return preg_replace
                    (
                        "/".Core_LibsysParams::getScreenPattern()."/",
                        "\\1_\\2",
                        $Param
                    );
            case "screenparam":
                return preg_replace
                    (
                        "/".Core_LibsysParams::getScreenParamPattern()."/",
                        "\\1_\\2",
                        $Param
                    );
            case "load":
                return preg_replace
                    (
                        "/".Core_LibsysParams::getLoadPattern()."/",
                        "\\1",
                        $Param
                    );
            case "loadparam": 
                return preg_replace
                    (
                        "/".Core_LibsysParams::getLoadParamPattern()."/",
                        "\\1",
                        $Param
                    );
        };
        return null;
    }
    
    protected function getSuperName($Param, $Type="superparam")
    {
        return $this->getGroupName($Param, $Type);
    }
    
    protected function getScreenName($Param, $Type="screenparam")
    {
        return $this->getGroupName($Param, $Type);
    }
    
    protected function getModNumber($Param, $Type="loadparam")
    {
        return $this->getGroupName($Param, $Type);
    }
    
    
    
    
    
    
    protected function getParamName($Param, $Type="loadparam")
    {
        switch ( strtolower($Type) )
        {
            
            case "loadparam": 
                return preg_replace
                    (
                        "/".Core_LibsysParams::getLoadParamPattern()."/",
                        "\\2",
                        $Param
                    );
            case "screenparam": 
                return preg_replace
                    (
                        "/".Core_LibsysParams::getScreenParamPattern()."/",
                        "\\3",
                        $Param
                    );
            case "superparam":
                return preg_replace
                    (
                        "/".Core_LibsysParams::getSuperParamPattern()."/",
                        "\\2",
                        $Param
                    );
        };
        return null;
    }
    

    
    
    
    
    
    protected function getParameters($RequestedGroupName, $AllParams, $Type)
    {
        switch ( strtolower($Type) )
        {
            
            case "load":
            case "loadparam": 
                return $this->getModParams($RequestedGroupName, $AllParams);
            case "screen":
            case "screenparam": 
                return $this->getScreenParams($RequestedGroupName, $AllParams);
            case "super":
            case "superparam":
                return $this->getSuperParams($RequestedGroupName, $AllParams);
        };
        return null;
    }
    
    protected function getModParams($RequestedModNumber, $AllParams)
    {
        $Params = array();
        foreach ( $AllParams as $Name => $Value )
        {
            //check if the name is appropiate
            if 
            ( 
                preg_match
                (
                    "/".Core_LibsysParams::getLoadParamPattern()."/",
                    $Name
                )
            )
            {
                //get the modnumber of the param
                $ModNumber = $this->getModNumber($Name);
                //check if the param has the requested modnumber
                if ( $RequestedModNumber === $ModNumber )
                {
                    $Params[$Name] = $Value;
                };
            };
        };
        return $Params;
    }

    protected function getScreenParams($RequestedScreenName, $AllParams)
    { 
        $Params = array();
        foreach ( $AllParams as $Name => $Value )
        {
            //check if the name is appropiate
            if 
            ( 
                preg_match
                (
                    "/".Core_LibsysParams::getScreenParamPattern()."/",
                    $Name
                )
            )
            { 
                //get the modnumber of the param
                $ScreenID = $this->getScreenName($Name);
                //check if the param has the requested modnumber
                if ( $RequestedScreenName === $ScreenID )
                {
                    $Params[$Name] = $Value;
                };
            };
        };
        return $Params;
    }
        
    protected function getSuperParams($RequestedSuperName, $AllParams)
    {
        $Params = array();
        foreach ( $AllParams as $Name => $Value )
        {
            //check if the name is appropiate
            if 
            ( 
                preg_match
                (
                    "/".Core_LibsysParams::getSuperParamPattern()."/",
                    $Name
                )
            )
            {
                //get the modnumber of the param
                $SuperName = $this->getSuperName($Name);
                //check if the param has the requested modnumber
                if ( $RequestedSuperName === $SuperName )
                {
                    $Params[$Name] = $Value;
                };
            };
        };
        return $Params;
    }




}

 ?>