<?php


final class Core_Main
{
 
    private $KernelName = "";
    private $KernelPart = "";
    private $KernelMode = "";
    private $KernelLevel = "";

    private $KernelClass = "";
        
    private $IsSystemThemeLoaded = false;
    private $IsSystemChecked = false;
    private $IsKernelLoaded = false;
    private $IsKernelChecked = false;
    
    private $Kernel = null;

    
    
    
    public function __construct()
    {
        //check if the needed data is given and comes from an initscript
        if 
        ( 
           !(
                isset($GLOBALS["KernelName"]) 
                && isset($GLOBALS["KernelPart"]) 
                && isset($GLOBALS["KernelMode"]) 
                && isset($GLOBALS["KernelLevel"])
                && defined("IS_INITSCRIPT_USED")
            ) 
        ) 
        {
            $this->throwFatalError();
        };
        $this->KernelName = $GLOBALS["KernelName"];
        $this->KernelPart = $GLOBALS["KernelPart"];
        $this->KernelMode = $GLOBALS["KernelMode"];
        $this->KernelLevel = $GLOBALS["KernelLevel"];
        $this->KernelClass = $this->KernelPart .
                             "_Kernels_" . $this->KernelName;
        
    }

    
    
    
    private function throwFatalError($Type)
    {
        die("eating engine fatal error: " . $Type . 
                " (Check your init scripts and kernel files)");
    }
    
    private function getLibFile($File)
    { 
        return SystemTheme_Paths::getFile("core", "lib", $File);
    }
    
    private function getLibClass($Class, $Iface=false)
    { 
        return SystemTheme_Classes::getFile
                ($Class, "Core", "lib", $Iface);
    }
    
    private function getKernelFile()
    {
        return SystemTheme_Classes::getFile
            ($this->KernelName, "Core", "Kernels");
    }
    
    
    
    public function checkSystem()
    {
        //check if the engine hasn't been loaded already
        if ( defined("IS_ENGINE_LOADED") )
        {
            $this->throwFatalError("system::engine_loaded");
        };
    
        //check for needed files
        if 
        ( 
           !(
                file_exists($this->getLibFile("Core_Kernel.class")) 
                && file_exists($this->getLibFile("Core_KernelBinding.iface"))
                && file_exists($this->getLibFile("Kernel.func"))
                && file_exists($this->getLibFile("__autoload.func"))
            ) 
        )
        {
            $this->throwFatalError("system::needed_files_existing");
        };
        
        //check if the given data is appropiate
        if ( !(is_string($this->KernelName) && is_integer($this->KernelLevel)) )
        {
            $this->throwFatalError("system::parameters_appropiate");
        };
        
        //check if the kernel exists
        if 
        ( 
            !file_exists($this->getKernelFile()) 
        )
        {
            $this->throwFatalError("system::kernel_class_exsisting");
        };
        
        //save that the system has been checked
        $this->IsSystemChecked = true;
       
        //return success
        return 0;
    }
    
    
    
    
    public function loadKernel()
    {
        //check if the system has been checked
        if( $this->IsSystemChecked !== true )
        {
            return 1;
        };
        
        //load the kernel files
        require_once ($this->getLibClass("Kernel"));
        require_once($this->getLibClass("KernelBinding", true));
        require_once ($this->getLibFile("Kernel.func"));
        require_once ($this->getKernelFile());
        require_once ($this->getLibFile("__autoload.func"));
        
        //create variable for the name of the class for the kernel
        $KernelClass = $this->KernelClass;
        
        //load the kernel
        $this->Kernel = new $KernelClass($this->KernelMode, $this->KernelLevel);
        
        //save that the kernel has been loaded
        $this->IsKernelLoaded = true;
        
        //return success
        return 0;
    }
    
    
    
    
    public function checkKernel()
    {
        //check if the kernel has been loaded
        if( $this->IsKernelLoaded !== true )
        {
            return 1;
        };
    
        //check if the needed classes are loaded
        if 
        ( 
           !(
                class_exists("Core_Kernel") 
                && class_exists($this->KernelClass)
            ) 
        )
        {
            $this->throwFatalError("kernel::needed_classes_loaded");
        };
        
        //check if the kernel class is a subclass of the class "core_Kernel"
        if
        ( 
           !(
                ($this->Kernel instanceof Core_Kernel) 
                && ($this->Kernel instanceof Core_KernelBinding) 
                //yes, you can use instanceof for interfaces
            )     
        )
        {
            $this->throwFatalError("kernel::right_subclass_for_kernel");
        };
        
        //save that the kernel has been checked
        $this->IsKernelChecked = true;
       
        //return success
        return 0;
    }
    
    
    
    
    public function getKernel()
    {
        //check if the kernel has been checked
        if( $this->IsKernelChecked !== true )
        {
            return 1;
        };
        
        //define a constant to proof that engine is loaded
        define("IS_ENGINE_LOADED", true);
        
        //set all class variables to default
        $this->IsSystemChecked = false;
        $this->IsKernelLoaded = false;
        $this->IsKernelChecked = false;
        
        //return the kernel
        return $this->Kernel;
    
    }
    
}
    
 ?>