<?php
/* EatingEngine E2 - A flexible, modular web framework
    Copyright (C)   2005   Thomas Gabor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


abstract class Core_Screen extends Core_Data
{
    protected $Part = null;
    protected $Module = null;
    protected $Style = null;
    
    /* design of style:
        $this->Style is an object of class Core_Style which 
            manages the templates
     */
    
    
    public final function __construct($Part, $Module, $Params=array())
    { 
        $this->data_useForced("GetVars", true);
    
        //save part and module
        $this->Part = $this->checkPart($Part);
        $this->Module = $this->checkModule($Module);
        //set default to style
        $this->Style = new Core_Null(array());
        //create an array with possible style sources
        $Styles = array();
        //add possible style by params
        if ( isset($Params["style"]) )
        {
            $Styles[] = $Params["style"];
        };
        //add possible style using the part specific configuration
        if ( $this->getStyleByConfig($this->Part->getName()) != false )
        {
            $Styles[] = $this->getStyleByConfig($this->Part->getName());
        };
        //add possible the style using the CORE configuration
        if ( $this->getStyleByConfig("Core") != false )
        {
            $Styles[] = $this->getStyleByConfig("Core");
        };
        //add the super-default style
        $Styles[] = "default";
        //save the first appropiate style
        foreach ( $Styles as $Number => $StyleName )
        { 
            $Style = new Core_Style($StyleName, $this, $this->Module);
            if ( $Style->data_Success_get() == true )
            {  
                $this->Style = $Style;
                break;
            };
        };
        //call the constructor of the specific screen if any
        if ( method_exists($this, "construct") == true )
        {
            $this->construct();
        };
    }
    
    public final function __destruct()
    {
        //call the destructor of the specific screen if any
        if ( method_exists($this, "destruct") == true )
        {
            $this->destruct();
        };
    }
    
    public final function output()
    {
        foreach ( $this->Style->getTmpls() as $Number => $Template )
        {
            //check if it is a valid template 
            if ( !is_object($Template) ) 
            { 
                continue;  
            };
            if ( !Core_LibClass::extendsClass($Template, "Core_Template") )
            {
                continue;
            };
            //check if the template has had success
            if ( !($Template->data_Success_get() === true) )
            {
                continue;
            };
            //check if there's a callback method of the screen
            if ( method_exists($this, "prepareOutput") )
            {
                ob_start(array($this, "prepareOutput"));
            }
            else
            {
                ob_start();
            };
            //check if there's a callback method of the template
            if ( method_exists($Template, "prepareOutput") )
            {
                ob_start(array($Template, "prepareOutput"));
            }
            else
            {
                ob_start();
            };
            //call the echo method of the template if it had success
            $Template->echoOutput($this, $this->Part, $this->Module);
            //end the output buffer (with template's callback)
            ob_end_flush();
            //end the output buffer (with screen's callback)
            ob_end_flush();
        };
    }
   
    /*
    private final function loadStyle($Style)
    { 
        //try to load the configuration file
        $StyleConfig = Kernel()->Core_Model()->get
            (
                $this->Part->getName(), 
                array("Tmpls", $Style, "style"),
                "Core_Table"
            );
        if ( !($StyleConfig instanceof Core_Null) )
        { 
            //load the configuration for this screen
            $ScreenConfig = $StyleConfig->get(get_class($this));
            if ( !empty($ScreenConfig) )
            {
                return new Core_Table($ScreenConfig);
            }; 
            //search for a default screen
            $ScreenConfig = $StyleConfig->get("default");
            if ( !empty($ScreenConfig) )
            {
                return new Core_Table($ScreenConfig);
            };  
        };
        return false;
    } */
    
    private final function getStyleByConfig($Part)
    {
        $ViewConfig = Kernel()->Core_Model()->get
            ($Part, array("Config", "View"), "Core_Table");
        if ( !($ViewConfig instanceof Core_Null) )
        {
            //load the configuration for this screen
            $ScreenConfig = $ViewConfig->get(get_class($this));
            if ( !empty($ScreenConfig) )
            {
                return $ScreenConfig["style"];
            }; 
            //search for a default screen
            $ScreenConfig = $ViewConfig->get("default");
            if ( !empty($ScreenConfig) )
            {
                return $ScreenConfig["style"];
            }; 
        };
        return false;
    }
    
    private final function checkPart($Part)
    {
        if ( is_object($Part) )
        {
            if ( Core_LibClass::extendsClass($Part, "Core_Part") == true )
            {
                return $Part;
            };
        };
        return new Core_Null();
    }
    
    private final function checkModule($Module)
    {
        if ( is_object($Module) )
        {
            if ( Core_LibClass::extendsClass($Module, "Core_Module") == true )
            {
                return $Module;
            };
        };
        return new Core_Null();
    }

}

 ?>