<?php
/* EatingEngine E2 - A flexible, modular web framework
    Copyright (C)   2005   Thomas Gabor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
    
class Core_Style extends Core_Data
{

    protected $Tmpls = array();
    protected $Style = ""; 
    protected $Screen = null;
    protected $Module = null;
    
   
    public function __construct($Style, $Screen, $Module)
    {
        $this->data_useForced("Success", true);
        $this->data_useForced("GetVars", true);
        
        if ( Core_LibClass::extendsClass($Module, "Core_Module") !== true )
        {
            return;
        }; 
        
        $this->Style = $Style;
        $this->Screen = $Screen;
        $this->Module = $Module;
        //try to load the configuration file
        $StyleConfig = Kernel()->Core_Model()->get
            (
                $this->Module->getPart()->getName(), 
                array("Tmpls", $Style, "style"),
                "Core_Table"
            );
        if ( !($StyleConfig instanceof Core_Null) )
        { 
            //load the configuration for this screen
            $ScreenConfig = $StyleConfig->get(get_class($Screen));
            if ( !empty($ScreenConfig) )
            { 
                $this->Tmpls = $this->loadStyle($ScreenConfig);
                $this->data_Success_succeed();
            }
            else
            {
                //search for a default screen
                $ScreenConfig = $StyleConfig->get("default");
                if ( !empty($ScreenConfig) )
                {
                    $this->Tmpls = $this->loadStyle($ScreenConfig);
                    $this->data_Success_succeed();
                };  
            };
        };
        
    }
    
    public function getAll()
    {
        return $this->Tmpls;
    }
    
    public function getTmpls()
    {
        return $this->Tmpls;
    }
    
    public function Screen()
    {
        return $this->Screen;
    }
    
    public function Module()
    {
        return $this->Module;
    }
    
    public function Style()
    {
        return $this->Style;
    }
    
    public function getStyle()
    {
        return $this->Style;
    }
    
    protected function loadStyle($ScreenConfig)
    {
        $Tmpls = array();
        //go through all defined templates
        foreach ( $ScreenConfig as $Number => $RawTemplateInfo )
        {
            $TemplateInfo = $this->handleTemplateInfo($RawTemplateInfo);
            if ( !is_array($TemplateInfo) )
            {
                return false;
            };
            $TemplateClass = $TemplateInfo["class"];
            if ( !Core_LibClass::extendsClass($TemplateClass, "Core_Template") )
            {
                continue;
            };
            $Template = new $TemplateClass 
                (
                    $this,
                    $TemplateInfo["part"], 
                    $TemplateInfo["style"],
                    $TemplateInfo["template"],
                    $TemplateInfo["params"]
                );
            $Tmpls[$Number] = $Template;
        };
        return $Tmpls;
    }
    
    protected function handleTemplateInfo($TemplateInfo)
    {
        $NewTemplateInfo = array();
        if ( !is_array($TemplateInfo) )
        {
            return false;
        };
        //TODO: make this construct easier
        //check the class
        if ( isset($TemplateInfo["class"]) )
        {
            $NewTemplateInfo["class"] = $TemplateInfo["class"];
        }
        else
        {
            return false;
        };
        //check the part
        if ( isset($TemplateInfo["part"]) )
        {
            $NewTemplateInfo["part"] = $TemplateInfo["part"];
        }
        else
        {
            $NewTemplateInfo["part"] = $this->Module->getPart()->getName();
        };
        //check the style
        if ( isset($TemplateInfo["style"]) )
        {
            $NewTemplateInfo["style"] = $TemplateInfo["style"];
        }
        else
        {
            $NewTemplateInfo["style"] = $this->Style;
        };
        //check the template
        if ( isset($TemplateInfo["template"]) )
        {
            if ( $TemplateInfo["template"] == "~" )
            {
                $NewTemplateInfo["template"] = $this->Module->getName();
            }
            else
            {
                $NewTemplateInfo["template"] = $TemplateInfo["template"];
            };
        }
        else
        {
            $NewTemplateInfo["template"] = "default";
        };
        //check the parameters
        if ( isset($TemplateInfo["params"]) )
        {
            $NewTemplateInfo["params"] = $TemplateInfo["params"];
        }
        else
        {
            $NewTemplateInfo["params"] = array();
        };
        return $NewTemplateInfo;
    }

    
}  

 ?>