<?php
/* EatingEngine E2 - A flexible, modular web framework
    Copyright (C)   2005   Thomas Gabor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA#
*/

class Core_Services_Load extends Core_StaticService
{
    private $OriginalModules = null;
    private $Modules = array();
    private $Parts = array();

    protected function construct()
    { 
        //get the parameters
        $Params = Kernel()->Core_Model()->get("Core", "Params", "Core_Table");
        //load the requested modules
        $this->loadModulesByConfig($Params->get("load"), 1);
        //backup the original modules
        $this->OriginalModules = $this->Modules;
        //get the parts
        $Parts = Kernel()->Core_Model()->get("Core", "AllParts", "Core_Table");
        //add the predefined data from each part
        foreach ( $Parts->getAll() as $Number => $Part )
        {
            //load the configuration and check it
            $Config = Kernel()->Core_Model()->get
                ($Part, array("Config", "Params"), "Core_Table");
            if ( $Config instanceof Core_Null )
            {
                continue;
            }; 
            //extract the part for the modules
            $LoadConfig = new Core_Table($Config->get("load"));
            //add parameters ahead of the others
            $this->loadModulesByConfig($LoadConfig->get("pre"), 0);
            //add parameters after of the others
            $this->loadModulesByConfig($LoadConfig->get("post"), 1);
        };
    }
    
    public function run()
    {
        foreach ( $this->Modules as $Number => $Module )
        { 
            $Module->getPart()->run();
            $Module->run();
        };
    }
    
    public function getRequestedModule($Number)
    {
        if ( !isset($this->OriginalModules[$Number]) )
        {
            return new Core_Null();
        };
        return $this->OriginalModules[$Number];
    }
    
    public function getAllRequestedModules()
    {
        return $this->OriginalModules;
    }
    
    
    /* OLD STUFF - NOT NEEDED
    
    
    public function getRequestedPartName($Number)
    {
        //TODO: build in good design!!
        $Module = $this->GetRequestedModule($Number);
        if ( $Module instanceof Core_Null )
        {
            return "";
        };
        return preg_replace
            (
                "/^" . Core_LibsysClasses::getPattern() . "$/", 
                "\\1",
                get_class($Module)
            );
    }
    
    
    
    public function getRequestedModuleName($Number)
    {
        //TODO: build in good design!!
        $Module = $this->GetRequestedModule($Number);
        if ( $Module instanceof Core_Null )
        {
            return "";
        };
        return preg_replace
            (
                "/^" . Core_LibsysClasses::getPattern() . "$/", 
                "\\3",
                get_class($Module)
            );
    }
    
    public function getRequestedFullName($Number)
    {
        return Core_LibsysParams::getMod
            (
                $this->getRequestedPartName($Number),
                $this->getRequestedModuleName($Number)
            );
    }
    
    public function getAllRequestedPartNames()
    {
        $ReturnData = array();
        foreach ( $this->OriginalModules as $Number => $ModuleData )
        {
            $ReturnData[$Number] = $this->getRequestedPartName($Number);
        };
        return $ReturnData;
    }
    
    
    
    public function getAllRequestedModuleNames()
    {
        $ReturnData = array();
        foreach ( $this->OriginalModules as $Number => $ModuleData )
        {
            $ReturnData[$Number] = $this->getRequestedModuleName($Number);
        };
        return $ReturnData;
    }
    
    public function getAllRequestedFullNames()
    {
        $ReturnData = array();
        foreach ( $this->OriginalModules as $Number => $ModuleData )
        {
            $ReturnData[$Number] = $this->getRequestedFullName($Number);
        };
        return $ReturnData;
    }
    
    */
    
    protected function getPartName($Module)
    {
        return preg_replace
            (
                "/".Core_LibsysParams::getModPattern()."/",
                "\\1",
                $Module
            );
    }
    
    protected function getModuleName($Module)
    {
        return preg_replace
            (
                "/".Core_LibsysParams::getModPattern()."/",
                "\\2",
                $Module
            );
    }
    
    
    protected function loadModule($FullModuleName, $Params=array())
    {
        //get the names
        $PartName = $this->getPartName($FullModuleName);
        $ModName = $this->getModuleName($FullModuleName);
        //get the classes
        $PartClass = 
            Core_LibsysClasses::getName($PartName, "Modules", "Main");
        $ModClass = 
            Core_LibsysClasses::getName($PartName, "Modules", $ModName);
        //load the part if not already done
        if ( !isset($this->Parts[$PartName]) )
        {
            if ( Core_LibClass::extendsClass($PartClass, "Core_Part") )
            {
                $this->Parts[$PartName] = new $PartClass();
            }
            else
            {
                return new Core_Null();
            };
        };
        //load the module
        if ( Core_LibClass::extendsClass($ModClass, "Core_Module") )
        {
            return new $ModClass
                ($this->Parts[$PartName], $Params); 
        }; 
        return new Core_Null();
    }
    
    
    protected function loadModulesByConfig($Config, $Append=1)
    {
        if ( !is_array($Config) )
        { 
            return false;
        };
        //go through the configuration
        foreach ( $Config as $Module => $Data )
        { 
            //check if it is a module 
            if ( !(isset($Data["value"]) && isset($Data["params"])) )
            {
                continue;  
            };
            if ( !is_array($Data["params"]) )
            {
                continue;  
            };
            //load the module
            $Module = $this->loadModule($Data["value"], $Data["params"]);
            //check if it really is a module
            if ( $Module instanceof Core_Null )
            {
                continue;
            };
            //save the module
            if ( $Append == 0 )
            {
                array_unshift
                    (
                        $this->Modules,  
                        $Module
                    );
            }
            else
            {
                array_push
                    (
                        $this->Modules,  
                        $Module
                    );
            };
        };
        return true;
    }
    
}



 ?>