<?php
/* EatingEngine E2 - A flexible, modular web framework
    Copyright (C)   2005   Thomas Gabor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

class Core_Services_Model extends Core_StaticService
{    
    
    protected function construct()
    {
        //$Test = new Core_Drivers_Driver("Core", "blah");
    }
    
    public function get
    (
        $Part, 
        $Source, 
        $Class="Core_Container",
        $Params=array()
    )
    {
        //$Part, $Source: define the source
        //$Class: class of the object which is generated to save the data
        //$Params: the parameters for the driver receiver
        
        //check if the receiver is correct
        if 
        ( 
           !Core_LibClass::implementsIface
                ($Class, "Core_DriverReceiverBindings")
        )
        {  
            return new Core_Null();
        };
        
        
        //load the driver
        $Driver = $this->getDriver($Part, $Source, $Class);
        
        //check the driver
        if ( $this->checkDriver($Driver) !== true )
        { 
            return new Core_Null(); 
        };
        
       
        return new $Class ($Driver, $Params);
    }
    
    public function set($Part, $Source, $Receiver)
    {
        //$Part, $Source: define the source
        //$Receiver: an instance of a driver-receiver data which contains
        //           the new data
        
        //check if the receiver is correct
        if 
        ( 
           !Core_LibClass::implementsIface
                ($Receiver, "Core_DriverReceiverBindings")
        )
        {  
            return false;
        };
        
        //load the driver
        $Driver = $this->getDriver($Part, $Source, get_class($Receiver));
        
        //check the driver
        if ( $this->checkDriver($Driver) !== true )
        { 
            return false; 
        };
        
        return $Driver->set($Receiver);
    }
    
    
    public function getDriver($Part, $Source, $Class="__special__value__")
    {
        if ( $Class != "__special__value__" )
        {
            //initiate the driver for the driver
            $SuperDriver = new Core_Drivers_Driver
                ($Part, $Source, array("Class" => $Class));
        }
        else
        {
            //initiate the driver for the driver
            $SuperDriver = new Core_Drivers_Driver
                ($Part, $Source);
        };
        //load the driver
        $Driver = $SuperDriver->getObject();
        
        //check the driver
        if ( $this->checkDriver($Driver) !== true )
        { 
            return new Core_Null(); 
        };
        
        //return the driver
        return $Driver;
    }
    
    public function checkDriver($Driver, $CheckSuccess=false)
    {
        //check if the driver really is an object
        if ( !is_object($Driver) )
        { 
            return false;
        };
        //check if the driver is correct
        if 
        (
           !(
                Core_LibClass::implementsIface($Driver, "Core_DriverBindings")
                && Core_LibClass::extendsClass($Driver, "Core_Driver")
            )
        ) 
        {
            return false;
        };
        //check if the driver has been successful (if demanded)
        if ( $CheckSuccess == true )
        {
            if ( $Driver->data_Success_get() !== true )
            { 
                return false;
            };
        };
        return true;
    }
   
}



 ?>