<?php

class Extras_Package
{

    private $Contents = array();
    
    protected $Part = "";
    protected $Sub = "";
    protected $Name = "";

    public final function __construct()
    {
        //set the default values for the namespaces by extracting the class name
        $Spaces = explode("_", get_class($this));
        $this->Part = $Spaces[0];
        $this->Sub = $Spaces[1];
        $this->Name = $Spaces[2];
        //call the specific constructor 
        if ( method_exists($this, "construct") )
        {
            $this->construct();
        };
    }
    
    public final function __destruct()
    {
        if ( method_exists($this, "destruct") )
        {
            $this->destruct();
        };
    }
    
    public final function install()
    {
        //check if the system is properly
        if ( $this->checkSystem() !== true )
        { 
            return false;
        };
        //call the specific method
        if ( method_exists($this, "prepareInstall") )
        {
            $this->prepareInstall();
        };
        
        //create array to save the success
        $Success = array();
        //save the contents
        foreach ( $this->Contents as $Number => $Content )
        {
            $Source = $Content["src"];
            $Destination = $Content["dest"];
            $Success[$Number] = $Destination->set($Source);
        };
        
        //call the specific method
        if ( method_exists($this, "completeInstall") )
        {
            return $this->completeInstall($Success);
        };
        return true; //assume that the install is not so complicated
    }
    
    public final function uninstall()
    {
        //check if the package is installed
        if ( $this->isInstalled() !== true )
        {
            return false;
        };
        //call the specific method
        if ( method_exists($this, "prepareUninstall") )
        {
            $this->prepareUninstall();
        };
        
        //create array to save the success
        $Success = array();
        //create an empty value as source
        $Source = Kernel()->Core_Model()->get("Core", "Null");
        //save the contents
        foreach ( $this->Contents as $Number => $Content )
        {
            $Destination = $Content["dest"];
            $Success[$Number] = $Destination->set($Source);
        };
        
        //call the specific method
        if ( method_exists($this, "completeUninstall") )
        {
            return $this->completeUninstall($Success);
        };
        return true; //assume that the uninstall is not so complicated
    }
     
    public function checkSystem()
    {
        return true;
    }
    
    public function isInstalled()
    {
        return false;
    }
    
    
    protected final function addContent($SourceName, $DestinationName)
    {
        //load the driver for the source
        $Source = Kernel()->Core_Model()->get
            (
                $this->Part,
                array($this->Sub, $this->Name, "src", $SourceName),
                "Core_Container"
            );
        //read in the destination
        if ( is_array($DestinationName) )
        {
            //read the part and the name
            $Part = $DestinationName[0];
            array_shift($DestinationName);
            $Name = $DestinationName;
        }
        else
        {
            $Part = $this->Part;
            $Name = $DestinationName;
        };
        $Destination = Kernel()->Core_Model()->getDriver($Part, $Name); 
        //check the source and destination
        if 
        ( 
            ($Source instanceof Core_Null) 
            || ($Destination instanceof Core_Null)
        )
        {
            return false;
        };
        //save the source and destination
        $this->Contents[] = array("src" => $Source, "dest" => $Destination);
        return true;
    }


}

 ?>