<?php
/* EatingEngine E2 - A flexible, modular web framework
    Copyright (C)   2005   Thomas Gabor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


class Extras_Services_Packages extends Core_RuntimeService
{

    protected $Config = array();

    public function __construct()
    {
        $this->Config = Kernel()->Core_Model()
            ->get("Extras", array("Config", "Packages"), "Core_Table");
    }

    
    public function installPackage($Package)
    {
        //check if installation is allowed
        if ( $this->Config->get("AllowInstallation") != true )
        {
            return false;
        };
        //check if it is a correct package
        if ( !$this->checkPackage($Package) )
        {
            return false;
        };
        //check if the package already is installed
        if ( $Package->isInstalled() === true )
        {
            return true;
        };
        //install the package (remember: Extras_Package does the tests)
        $Package->install();
        //return if the package really is installed now
        if ( $Package->isInstalled() === true )
        {
            return true;
        };
        return false;
    }
    
    public function reinstallPackage($Package)
    {
        //check if installation is allowed
        if ( $this->Config["AllowInstallation"] != true )
        {
            return false;
        };
        //check if it is a correct package
        if ( !$this->checkPackage($Package) )
        {
            return false;
        };
        //check if the package already is installed
        if ( $Package->isInstalled() !== true )
        {
            return false;
        };
        //install the package (remember: Extras_Package does the tests)
        $Package->install();
        //check if the package is still installed 
        if ( $Package->isInstalled() !== true )
        {
            return false;
        };
        return true;
    }
    
    public function uninstallPackage($Package)
    {
        //check if installation is allowed
        if ( $this->Config["AllowInstallation"] != true )
        {
            return false;
        };
        //check if it is a correct package
        if ( !$this->checkPackage($Package) )
        {
            return false;
        };
        //check if the package really is installed
        if ( $Package->isInstalled() !== true )
        {
            return false;
        };
        //uninstall the package (remember: Extras_Package does the tests)
        $Package->uninstall();
        //check if the package is still installed 
        if ( $Package->isInstalled() == true )
        {
            return false;
        };
        return true;
    }
    
    public function checkPackage($Package)
    {
        //check if it is a correct package
        if ( is_object($Package) === true )
        { 
            if 
            ( 
                Core_LibClass::extendsClass($Package, "Extras_Package") 
                === true 
            )
            { 
                return true;
            };
        };
        return false;
    }
    
    public function getPackage($Name, $Check=false)
    {
        $ClassName = Kernel()->Core_Model()->get
            ("Extras", array("Pkgs", $Name, $Name, "pkg"))->getText();
        if ( !class_exists($ClassName) )
        {
            return false;
        };
        $Package = new $ClassName();
        if ( $Check == true )
        {
            if ( $this->checkPackage($Package) == true )
            {
                return $Package;
            };
            return new Core_Null();
        }
        else
        {
            return $Package;
        };
    }
    
    public function install($Name)
    {
        return $this->installPackage($this->getPackage($Name));
    }
    
    public function restall($Name)
    {
        return $this->restallPackage($this->getPackage($Name));
    }
    
    public function uninstall($Name)
    {
        return $this->uninstallPackage($this->getPackage($Name));
    }
    
    public function check($Name)
    {
        return $this->checkPackage($this->getPackage($Name));
    }
}

?>